@@ -482,6 +482,36 @@ append_folder (CameraFilesystem *fs, const char *folder, GPContext *context)
         return (GP_OK);
 }
 
+#define X(x)											\
+	if (((x - old) >= 0) && ((x - old) < prevcount)) {					\
+		x = new + (x - old);								\
+	}
+
+
+/* When we realloc the files[] array, we need to fix up the
+ * lru pointers. 
+ * in libgphoto2 2.5 we will use normal linked lists instead of
+ * this hack.
+ */
+static void
+fixup_lru_chain (CameraFilesystem *fs, CameraFilesystemFile *old,
+	CameraFilesystemFile *new, int prevcount) {
+	CameraFilesystemFile	*x;
+
+	if (!fs->lru_first) return;
+	if (new == old) return;
+
+	X(fs->lru_first);
+	X(fs->lru_last);
+
+	x = fs->lru_first;
+	do {
+		X (x->lru_next);
+		X (x->lru_prev);
+		x = x->lru_next;
+	} while (x && (x != fs->lru_last));
+}
+
 static int
 append_file (CameraFilesystem *fs, int x, CameraFile *file, GPContext *context)
 {
@@ -498,6 +528,7 @@ append_file (CameraFilesystem *fs, int x, CameraFile *file, GPContext *context)
 		CHECK_MEM (new = realloc (fs->folder[x].file,
 					sizeof (CameraFilesystemFile) *
 						(fs->folder[x].count + 1)));
+	fixup_lru_chain (fs, fs->folder[x].file, new, fs->folder[x].count);
 	fs->folder[x].file = new;
 	fs->folder[x].count++;
 	memset (&(fs->folder[x].file[fs->folder[x].count - 1]), 0,
@@ -711,6 +742,7 @@ gp_filesystem_append (CameraFilesystem *fs, const char *folder,
 		CHECK_MEM (new = realloc (fs->folder[x].file,
 					sizeof (CameraFilesystemFile) *
 					(fs->folder[x].count + 1)));
+	fixup_lru_chain (fs, fs->folder[x].file, new, fs->folder[x].count);
 	fs->folder[x].file = new;
 	fs->folder[x].count++;
 	memset (&(fs->folder[x].file[fs->folder[x].count - 1]), 0,
@@ -792,6 +824,7 @@ delete_file (CameraFilesystem *fs, int x, int y)
 	/* Get rid of the last one */
 	new_fip = realloc (fs->folder[x].file,
 		sizeof (CameraFilesystemFile) * (fs->folder[x].count));
+	fixup_lru_chain (fs, fs->folder[x].file, new_fip, fs->folder[x].count);
 	if (!fs->folder[x].count || (fs->folder[x].count && new_fip))
 		fs->folder[x].file = new_fip;
 
@@ -1998,7 +2031,7 @@ gp_filesystem_lru_free (CameraFilesystem *fs)
 		fs->lru_first->lru_prev = fs->lru_first;
 	else
 		fs->lru_last = NULL;
-
+	ptr->lru_next = ptr->lru_prev = NULL;
 	/* Free its content. */
 	if (ptr->normal) {
 		CR( gp_file_get_data_and_size (ptr->normal, NULL, &size));
@@ -2018,7 +2051,6 @@ gp_filesystem_lru_free (CameraFilesystem *fs)
 		gp_file_unref (ptr->audio);
 		ptr->audio = NULL;
 	}
-	ptr->lru_next = ptr->lru_prev = NULL;
 	return (GP_OK);
 }
 
